/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Peak.hpp
 *
 *  Created on: 2010/12/13
 *      Author: HARK Development Team
 */

#ifndef PEAK_HPP_
#define PEAK_HPP_

#include <vector>
#include <boost/shared_ptr.hpp>
#include <Channel.hpp>
#include <LRSignal.hpp>
#include <Band.hpp>
#include <BandAndPower.hpp>

namespace NSHarkBinaural
{

class Peak
{
public:
    typedef std::vector<BandAndPower> PeakVector;
    typedef boost::shared_ptr<PeakVector> PeakVectorPtr;
    typedef ChannelHolder<PeakVectorPtr, PeakVectorPtr> LRPeaks;
    typedef boost::shared_ptr<LRPeaks> LRPeaksPtr;

    /**
     * 信号のピークを取得する.
     * lowerからupperまでの周波数の中で、ルームノイズよりthresDB以上
     * 大きなパワーの信号をピークと判断し、返す。
     * 返される配列にはピークと判断された信号のみが含まれる。
     *
     * @param bgnoise ルームノイズのパワースペクトル
     * @param lrpower 左右信号のパワースペクトル（要素はデシベル表示）
     * @param lower 考慮する信号の最小周波数
     * @param upper 考慮する信号の最大周波数
     * @param bi 周波数バンド情報
     * @param thresDB ピークを判定する際の、ルームノイズに対する閾値
     * @return ピークの信号のリスト
     */
    static LRPeaksPtr
    GetPeaks(
        const LRUnspectra::ptr_t bgnoise,
        const LRUnspectra::ptr_t lrpower,
        const Band& lower,
        const Band& upper,
        const BandInformation& bi,
        //Unspectrum::ElementType minDB = -1000.0,
        Unspectrum::ElementType thresDB = 15.0);

};

}

#endif /* PEAK_HPP_ */
